package cz.muni.fi.pv168.graves;

import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import org.apache.commons.dbcp.BasicDataSource;
import java.sql.SQLException;
import java.util.HashSet;
import javax.sql.DataSource;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 * This is a test for cemetery manager.
 * There are implemented two testing methods.
 *
 * 
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @author Petr Adamek (xadamek2@fi.muni.cz)
 */
public class CemeteryManagerTest {

    private CemeteryManagerImpl manager;
    private DataSource ds;

    public CemeteryManagerTest() {
    }

    /**
     * Prepares datasource, it is using BasicDataSource from commons-dbcp
     * @return datasource
     * @throws SQLException
     */
    private static DataSource prepareDataSource() throws SQLException {
        BasicDataSource ds = new BasicDataSource();
        //ds.setUrl("jdbc:derby://localhost:1527/graveyard3");
        //ds.setUsername("death");
        //ds.setPassword("death");
        //ds.setDriverClassName("org.apache.derby.jdbc.ClientDriver");

        //we will use in memory database
        ds.setUrl("jdbc:derby:memory:grave-test;create=true");
        return ds;
    }

    /**
     * We must prepare datasource and create tables before each test.
     * @throws SQLException
     */
    @Before
    public void setUp() throws SQLException {
        ds = prepareDataSource();
        DBUtils.createTables(ds);
        manager = new CemeteryManagerImpl();
        manager.setDataSource(ds);

    }

    /**
     * We wan't to have the database clean after executing each test.
     * @throws SQLException
     */
    @After
    public void tearDown() throws SQLException {
        DBUtils.dropTables(ds);
    }

    /**
     * Test of getAllGraves method, of class CemeteryManager.
     */
    @Test
    public void testGetAllGraves() {
                fail("Test not implemented");

    }

    /**
     * Test of getGraveById method, of class CemeteryManager.
     */
    @Test
    public void testGetGraveById() {
        fail("Test not implemented");


    }

    /**
     * Test of addGrave method, of class CemeteryManager.
     * Tests correct adding
     */
    @Test
    public void testAddGrave() {
        Grave grave = createSampleGrave(2);
        //we try to add a correct grave
        manager.addGrave(grave);
        Long graveId = grave.getId();
        //now id should not be null
        assertNotNull(graveId);
        //we try to get the object back
        Grave result = manager.getGraveById(graveId);
        //should be the same
        assertEquals(grave, result);

        assertDeepEquals(grave, result);

        List<Grave> expected = Arrays.asList(grave);
        List<Grave> actual = manager.getAllGraves();

        Collections.sort(actual, idComparator);
        Collections.sort(expected, idComparator);

        assertEquals(expected, actual);
        assertDeepEquals(expected, actual);

        //we try to add a grave with capacity 1
        //we try to add a correct grave
        Grave grave1 = createSampleGrave(1);
        manager.addGrave(grave1);
        graveId = grave1.getId();
        result = manager.getGraveById(graveId);
        //should be the same
        assertEquals(grave1, result);

        assertDeepEquals(grave1, result);

        expected = Arrays.asList(grave, grave1);
        actual = manager.getAllGraves();

        Collections.sort(actual, idComparator);
        Collections.sort(expected, idComparator);

        assertEquals(expected, actual);
        assertDeepEquals(expected, actual);
    }

    /**
     * Test of addGrave method, of class CemeteryManager.
     * Tests incorrect adding
     */
    @Test
    public void testAddGraveErrors() {
        Grave grave = createSampleGrave(2);
        //when we try to add null, we should get an exception
        try {
            manager.addGrave(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        grave.setId(5l);
        //when we try to add a grave with set id, we should get an exception
        try {
            manager.addGrave(grave);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        //when we try to add a grave with negative capacity, we should get an exception
        grave.setId(null);
        grave.setCapacity(-1);
        try {
            manager.addGrave(grave);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        //when we try to add a grave with zero capacity, we should get an exception
        grave.setId(null);
        grave.setCapacity(0);
        try {
            manager.addGrave(grave);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
    }

    /**
     * Test of deleteGrave method, of class CemeteryManager.
     */
    @Test
    public void testDeleteGrave() {
        fail("Test not implemented");
    }

    /**
     * Test of updateGrave method, of class CemeteryManager.
     */
    @Test
    public void testUpdateGrave() {
        fail("test not implemented");
    }

    private static Grave createSampleGrave(int capacity) {
        Grave grave = new Grave();
        grave.setCapacity(capacity);
        grave.setBodies(new HashSet<Body>());
        grave.setColumn(1);
        grave.setRow(1);
        return grave;
    }

    private void assertDeepEquals(List<Grave> expectedList, List<Grave> actualList) {
        for (int i = 0; i < expectedList.size(); i++) {
            Grave expected = expectedList.get(i);
            Grave actual = actualList.get(i);
            assertDeepEquals(expected, actual);
        }
    }

    private void assertDeepEquals(Grave expected, Grave actual) {
        assertEquals(expected.getCapacity(), actual.getCapacity());
        assertEquals(expected.getRow(), actual.getRow());
        assertEquals(expected.getColumn(), actual.getColumn());
    }
    private static Comparator<Grave> idComparator = new Comparator<Grave>() {

        public int compare(Grave o1, Grave o2) {
            return Long.valueOf(o1.getId()).compareTo(Long.valueOf(o2.getId()));
        }
    };
}
